"use strict";
exports.__esModule = true;
var ts = require("typescript");
var _ = require("lodash");
var TsUtil = require("../util/tsnode.parser");
var IslandBaseService = /** @class */ (function () {
    function IslandBaseService(program) {
        this.enums = {};
        // Includes interface, class, type declaration
        this.declarations = {};
        this.program = program;
        if (!this.program)
            return;
        this.sourceFiles = this.program.getSourceFiles();
        this.typeChecker = this.program.getTypeChecker();
        this.parseFiles();
        // console.log(JSON.stringify(this.enums, null, 2));
        // console.log(JSON.stringify(this.declarations, null, 2));
    }
    IslandBaseService.prototype.getInterfaceDeclaration = function (interfaceName, regexOptions) {
        if (regexOptions === void 0) { regexOptions = ''; }
        var matched = interfaceName.match(/\.([A-z0-9]*)\)?$/);
        if (matched && matched[1]) {
            return this.getValue("\\." + matched[1] + "$", regexOptions);
        }
        return [];
    };
    IslandBaseService.prototype.getValue = function (keyword, regexOptions) {
        if (regexOptions === void 0) { regexOptions = ''; }
        var testRegex = new RegExp(keyword, regexOptions);
        var values = [];
        _.forEach(this.declarations, function (v, k) {
            if (testRegex.test(k)) {
                values.push(v);
            }
        });
        _.forEach(this.enums, function (v, k) {
            if (testRegex.test(k)) {
                values.push(v);
            }
        });
        return values;
    };
    IslandBaseService.prototype.parseFiles = function () {
        for (var _i = 0, _a = this.sourceFiles; _i < _a.length; _i++) {
            var sf = _a[_i];
            if (!/island-base/.test(sf.fileName))
                continue;
            this.visit(sf);
        }
    };
    IslandBaseService.prototype.visit = function (node) {
        var _this = this;
        // console.log(TsUtil.getNodeType(node));
        switch (node.kind) {
            case ts.SyntaxKind.TypeAliasDeclaration:
            case ts.SyntaxKind.ClassDeclaration:
            case ts.SyntaxKind.InterfaceDeclaration:
                {
                    var nodeName = TsUtil.getDeepName(node);
                    this.declarations[nodeName.join('.')] = TsUtil.getNodeText(node);
                }
                break;
            case ts.SyntaxKind.EnumDeclaration:
                {
                    var nodeName = TsUtil.getDeepName(node);
                    this.enums[nodeName.join('.')] = TsUtil.getNodeText(node);
                }
                break;
        }
        ts.forEachChild(node, function (childNode) { return _this.visit(childNode); });
    };
    return IslandBaseService;
}());
exports.IslandBaseService = IslandBaseService;
