"use strict";
exports.__esModule = true;
var _ = require("lodash");
function padEnd(srcString, targetLength, padString) {
    targetLength = targetLength >> 0; //floor if number or convert non-number to 0;
    padString = String((typeof padString !== 'undefined' ? padString : ' '));
    if (srcString.length > targetLength) {
        return String(srcString);
    }
    else {
        targetLength = targetLength - srcString.length;
        if (targetLength > padString.length) {
            padString += padString.repeat(targetLength / padString.length); //append to original to ensure we are longer than needed
        }
        return String(srcString) + padString.slice(0, targetLength);
    }
}
;
var padArrayWithLength = function (objects, lengths, padString) {
    if (padString === void 0) { padString = ' '; }
    var items = [];
    _.forEach(objects, function (nodeInfo) {
        var item = [];
        var idx = 0;
        _.forEach(lengths, function () {
            var value = typeof nodeInfo[idx] === 'string' ? nodeInfo[idx] : '';
            item.push(padEnd(value, lengths[idx], padString));
            idx++;
        });
        items.push(item);
    });
    return items;
};
exports.createTableFromMdText = function (objects) {
    if (!objects || Object.keys(objects).length === 0)
        return '';
    var tempKeys = [];
    _.forEach(objects, function (nodeInfo) { return _.forEach(nodeInfo, function (v, key) { return tempKeys.push(key); }); });
    var keys = _.uniq(tempKeys);
    var lengths = new Array(keys.length).fill(1);
    var items = [];
    _.forEach(objects, function (nodeInfo) {
        var item = [];
        var idx = 0;
        _.forEach(keys, function (key) {
            item.push(nodeInfo[key] || '');
            lengths[idx] = 1; //Math.max(lengths[idx], item[idx].length);
            ++idx;
        });
        items.push(item);
    });
    var mdTextPad = [];
    var tableHeader = padArrayWithLength([keys], lengths);
    var tableHorizon = padArrayWithLength([lengths], lengths, '-');
    var tableBody = padArrayWithLength(items, lengths, ' ');
    var table = tableHeader.concat(tableHorizon).concat(tableBody);
    var stringArray = _.map(table, function (v) { return '| ' + v.join(' | ') + ' |'; });
    var envString = stringArray.join('\n');
    return envString;
};
exports.createMdText = function (objects) {
    var str = [];
    if (Array.isArray(objects)) {
        str.push('```javascript');
        _.forEach(objects, function (value, key) {
            str.push("" + value);
        });
        str.push('```');
        return str.join('\n');
    }
    if (!objects || Object.keys(objects).length === 0)
        return '';
    _.forEach(objects, function (nodeInfo) {
        if (nodeInfo.Name) {
            str.push("### " + nodeInfo.Name);
        }
        str.push('```javascript');
        _.forEach(nodeInfo, function (value, key) {
            if (value === undefined || value.length === 0)
                return;
            if (key === 'Name')
                return;
            str.push("" + value);
        });
        str.push('```');
    });
    return str.join('\n');
};
exports.replaceSection = function (srcText, sectionName, replaceWith) {
    if (!replaceWith)
        return srcText;
    var envMatchedSection = srcText.match(new RegExp("\n(#+) " + sectionName));
    var sectionTitle = '\n\n## ' + sectionName;
    var prevSection = srcText;
    var nextSection = '';
    // 기본적으로 제일 뒤에 붙임.
    // 가운데에 # Environment Variables 가 있으면 replace.
    if (envMatchedSection) {
        sectionTitle = envMatchedSection[0];
        var sectionIndent = envMatchedSection[1];
        var startIdx = envMatchedSection.index;
        var nextMatchedSection = srcText.substring(startIdx + sectionTitle.length).match(new RegExp("\n" + sectionIndent + " [A-zs]+"));
        prevSection = srcText.substring(0, startIdx);
        if (nextMatchedSection) {
            var endIdx = nextMatchedSection ? nextMatchedSection.index : undefined;
            nextSection = srcText.substring(startIdx + endIdx + sectionTitle.length);
        }
    }
    return prevSection + sectionTitle + '\n\n' + replaceWith + '\n\n' + nextSection;
};
