"use strict";
exports.__esModule = true;
var path = require("path");
var ts = require("typescript");
exports.getSourceFile = function (anyTsObject) {
    if (anyTsObject.parent)
        return exports.getSourceFile(anyTsObject.parent);
    if (Array.isArray(anyTsObject) && anyTsObject.length) {
        return exports.getSourceFile(anyTsObject[0]);
    }
    return anyTsObject;
};
exports.findNode = function (node, kind) {
    var found = [];
    ts.forEachChild(node, function (childNode) {
        if (!kind || childNode.kind === kind) {
            found.push(childNode);
        }
    });
    return found;
};
exports.getNodeName = function (node) {
    if (!node)
        return undefined;
    var anyNode = node;
    return anyNode.escapedName
        || (anyNode.name && anyNode.name.escapedText)
        || undefined;
};
exports.getDeepName = function (node, names) {
    if (names === void 0) { names = []; }
    // if there is no parent, that node is source file
    if (node.parent) {
        var nodeName = exports.getNodeName(node);
        if (nodeName) {
            names.unshift(nodeName);
        }
        exports.getDeepName(node.parent, names);
    }
    else {
        var nodeName = path.basename(node.path);
        if (nodeName) {
            names.unshift(nodeName.replace(/\.ts/, ''));
        }
    }
    return names;
};
exports.getNodeType = function (node) {
    var type = node.type || node;
    if (!type || !type.kind) {
        console.warn("some node has no type declaration. treat it as any");
        return 'Treat It As Any';
    }
    return ts.SyntaxKind[type.kind].replace('Keyword', '');
};
exports.getNodeText = function (node) {
    if (!node || node.pos === undefined || node.end === undefined)
        return '';
    var sourceFile = exports.getSourceFile(node);
    if (node.getText)
        return node.getText(sourceFile);
    var text = sourceFile.getText();
    return text.substring(node.pos, node.end);
};
exports.getNodeLeadingComments = function (node) {
    var sourceFile = node.getSourceFile();
    var comments = ts.getLeadingCommentRanges(sourceFile.getText(), node.getFullStart());
    if (!comments) {
        return '';
    }
    var texts = [];
    for (var _i = 0, comments_1 = comments; _i < comments_1.length; _i++) {
        var comment = comments_1[_i];
        texts.push(sourceFile.getText().substring(comment.pos, comment.end));
    }
    return texts.length ? texts.join(' ') : '';
};
exports.getNodeDecorators = function (node) {
    var decorators = node.decorators;
    if (!decorators) {
        return undefined;
    }
    var texts = [];
    for (var _i = 0, decorators_1 = decorators; _i < decorators_1.length; _i++) {
        var decorator = decorators_1[_i];
        texts.push(exports.getNodeText(decorator));
    }
    return texts.length ? texts : undefined;
};
exports.getNodeDefaultValue = function (node) {
    var text = exports.getNodeText(node);
    var matched = text.match(/\=\s*(.+)\;/);
    return matched ? matched[1] : undefined;
};
exports.parseEnvDecoratorText = function (text) {
    var legacyKeys = text.match(/legacyKeys\:\s*\[([A-z\_\s\,\'\"]+)\]/);
    return {
        legacyKeys: legacyKeys ? legacyKeys[1] : undefined,
        required: false
    };
};
