import * as ts from 'typescript';
import * as _ from 'lodash';
import * as TsUtil from '../util/tsnode.parser';

export class IslandBaseService {
  private program: ts.Program;
  private typeChecker: ts.TypeChecker;
  private sourceFiles: ReadonlyArray<ts.SourceFile>;

  public enums: any = {};
  // Includes interface, class, type declaration
  public declarations: any = {};

  constructor(program: ts.Program) {
    this.program = program;
    if (!this.program) return;

    this.sourceFiles = this.program.getSourceFiles();
    this.typeChecker = this.program.getTypeChecker();

    this.parseFiles();

    // console.log(JSON.stringify(this.enums, null, 2));
    // console.log(JSON.stringify(this.declarations, null, 2));
  }

  public getInterfaceDeclaration(interfaceName: string, regexOptions: string = '') {
    const matched = interfaceName.match(/\.([A-z0-9]*)\)?$/);
    if (matched && matched[1]) {
      return this.getValue(`\\.${matched[1]}$`, regexOptions);
    }

    return [];
  }
  
  public getValue(keyword: string, regexOptions: string = '') {
    const testRegex = new RegExp(keyword, regexOptions);
    const values = [];
    _.forEach(this.declarations, (v, k) => {
      if (testRegex.test(k)) {
        values.push(v);
      }
    });
    _.forEach(this.enums, (v, k) => {
      if (testRegex.test(k)) {
        values.push(v);
      }
    });
    return values;
  }

  private parseFiles() {
    for (const sf of this.sourceFiles) {
      if (!/island-base/.test(sf.fileName)) continue;
      this.visit(sf);
    }
  }

  private visit(node: ts.Node) {
    // console.log(TsUtil.getNodeType(node));
    switch (node.kind) {
      case ts.SyntaxKind.TypeAliasDeclaration:
      case ts.SyntaxKind.ClassDeclaration:
      case ts.SyntaxKind.InterfaceDeclaration:
        {
          const nodeName = TsUtil.getDeepName(node);
          this.declarations[nodeName.join('.')] = TsUtil.getNodeText(node);
        }
        break;
      case ts.SyntaxKind.EnumDeclaration:
        {
          const nodeName = TsUtil.getDeepName(node);
          this.enums[nodeName.join('.')] = TsUtil.getNodeText(node);
        }
        break;
    }

    ts.forEachChild(node, (childNode) => this.visit(childNode));
  }
}