import * as ts from 'typescript';
import * as _ from 'lodash';
import * as fs from 'fs';
import { default as Confluency } from 'confluency';

// https://developer.atlassian.com/server/confluence/confluence-rest-api-examples/
export class ConfluenceUploader {
  // gitToken: string = 'pzNr-px6qbPma3FXESSV';
  private confluency = new Confluency({
    host: 'http://krconfluence.ea.com:8090',
    context: '',
    username: 'eass-build',
    password: 'Welcome2ea!!'
  });

  private getNextVersion(page: any) {
    if (!page || !page.version || !page.version.number) return 2;
    return page.version.number + 1;
  }

  private getIslandCode(projectName: string, islandCodes: any) {
    if (!islandCodes) return '';

    const matched = JSON.stringify(islandCodes).match(new RegExp(`[^A-z]${projectName}[\\s=]+([0-9]+)`));

    if (!matched) return '';

    return matched[1];
  }

  private errorCodeToHtml(projectInfo: any, errorCodes: any, islandCodes: any) {
    const projectName = projectInfo.name.toUpperCase().replace('-ISLAND', '').replace('-', '_');
    const islandCode = this.getIslandCode(projectName, islandCodes);

    let referenceBody = [];
    // enum IslandCode
    // enum ERROR
    referenceBody.push(`${projectInfo.name}-${projectInfo.version}`);
    referenceBody.push('<div class="table-wrap"><table class="confluenceTable">');
    referenceBody.push('<colgroup><col/><col/><col/><col/><col/></colgroup><tbody>');
    _.forEach(errorCodes, (code) => {
      const matched = code.match(/([A-z0-9]+)[\s=]+([0-9]+)/);
      if (matched) {
        referenceBody.push(`<tr>
        <td class="confluenceTd">${islandCode}${matched[2].padStart(5, '0')}</td>
        <td class="confluenceTd">${projectInfo.name}</td>
        <td class="confluenceTd">${matched[1]}</td>
        <td class="confluenceTd">${islandCode}</td>
        <td class="confluenceTd">${matched[2]}</td>
        </tr>`);
      }
    });
    referenceBody.push('</tbody></table></div>');

    // <div class="table-wrap"><table class="confluenceTable">
    // <colgroup><col/><col/></colgroup>
    // <tbody>
    // <tr><td class="confluenceTd">E0001_TEMP</td><td class="confluenceTd">1</td>
    // </tbody></table></div>
  //     referenceBody.push('<ac:structured-macro ac:name="toc" ac:schema-version="1" ac:macro-id="92e1ce42-8f5d-4ec5-a87a-cc567471a516" />');
  //     for (const methodType in reference[branch]) {
  //       referenceBody.push(`<h1>${methodType}</h1>`);
  //       for (const methodName in reference[branch][methodType]) {
  //         referenceBody.push(`<h2>${methodName}</h2>`);
  //         if (!reference[branch][methodType][methodName].decorators || reference[branch][methodType][methodName].decorators.length < 1) continue;
  //         reference[branch][methodType][methodName].decorators.sort();
  //         referenceBody.push(...reference[branch][methodType][methodName].decorators);
  //       }
  //     }
    // console.log(referenceBody);
    return referenceBody.join('');
  }

  public async uploadErrorCodes(projectInfo: any, errorCodes: any, islandCodes: any): Promise<void> {
    const parentSpace = 'ACT';
    const parentPageId = 66766730;

    const title = `${projectInfo.name}-${projectInfo.version}`;
    const pages = await this.confluency.getChildren(parentPageId, { all: true, expand: ['version'] });
    const targetPage = _.find(pages, (p) => { return p.title === title; });

    try {
      if (targetPage) {
        this.confluency.update({
          space: parentSpace,
          id: targetPage.id,
          version: this.getNextVersion(targetPage),
          title,
          content: this.errorCodeToHtml(projectInfo, errorCodes, islandCodes),
          parent: parentPageId
        });
      } else {
        this.confluency.create({
          space: parentSpace,
          title,
          content: this.errorCodeToHtml(projectInfo, errorCodes, islandCodes),
          parent: parentPageId
        });
      }
      console.log(`[Confluence Updated] ${title}`);
    } catch (err) {
      console.error(`[Confluence Update Error] ${err}`);
    }
    // console.log(data);
    return;
  }

  // async makeReference(projectInfo: any) {
  //   const { name, id, http_url_to_repo } = projectInfo;
  //   const reference: any = {};
  //   await this.gitClone(http_url_to_repo);
  //   const branches = (await this.getBranches(id)).map(info => info['name']);
  //   if (branches.length < 1) return;
  //   for (const branch of branches) {
  //     reference[branch] = {};
  //     console.log(`${name} - ${branch}`);
  //     await this.gitCheckout(name, branch);
  //     await this.getDecorator(`${this.sourcePath}/${name}/src/controller`, reference[branch]);
  //   }

  //   const projectPageId: number = await this.createPage(this.confluence.ancestors, `[Reference] ${name}`, '');
  //   for (const branch in reference) {
  //     if (!Object.keys(reference[branch])) continue;
  //     const referenceBody = [];
  //     referenceBody.push('<ac:structured-macro ac:name="toc" ac:schema-version="1" ac:macro-id="92e1ce42-8f5d-4ec5-a87a-cc567471a516" />');
  //     for (const methodType in reference[branch]) {
  //       referenceBody.push(`<h1>${methodType}</h1>`);
  //       for (const methodName in reference[branch][methodType]) {
  //         referenceBody.push(`<h2>${methodName}</h2>`);
  //         if (!reference[branch][methodType][methodName].decorators || reference[branch][methodType][methodName].decorators.length < 1) continue;
  //         reference[branch][methodType][methodName].decorators.sort();
  //         referenceBody.push(...reference[branch][methodType][methodName].decorators);
  //       }
  //     }
  //     await this.createPage(projectPageId, `[Reference] ${name} > ${branch}`, `<p>${referenceBody.join('</p><p>').replace(/\n/g, "")}</p>`);
  //   }
  // }

  // async run() {
  //   const gitGroupCode: number = 75;
  //   await this.initDir();
  //   const projects = await this.getProjects(gitGroupCode);
  //   //projects.map(project => this.removeAllPages(project));
  //   projects.map(project => this.makeReference(project));
  // }
}
