import * as _ from 'lodash';

function padEnd(srcString, targetLength, padString) {
  targetLength = targetLength >> 0; //floor if number or convert non-number to 0;
  padString = String((typeof padString !== 'undefined' ? padString : ' '));
  if (srcString.length > targetLength) {
      return String(srcString);
  }
  else {
      targetLength = targetLength - srcString.length;
      if (targetLength > padString.length) {
          padString += padString.repeat(targetLength / padString.length); //append to original to ensure we are longer than needed
      }
      return String(srcString) + padString.slice(0, targetLength);
  }
};

const padArrayWithLength = (objects: Object, lengths: Number[], padString = ' ') => {
  const items = [];
  _.forEach(objects, (nodeInfo) => {
    const item = [];
    let idx = 0;
    _.forEach(lengths, () => {
      const value = typeof nodeInfo[idx] === 'string' ? nodeInfo[idx] : '';
      item.push(padEnd(value, lengths[idx], padString));
      idx++;
    });
    items.push(item);
  });
  return items;
};

export const createTableFromMdText = (objects: Object) => {
  if (!objects || Object.keys(objects).length === 0) return '';
  
  let tempKeys = [];

  _.forEach(objects, (nodeInfo) => _.forEach(nodeInfo, (v, key) => tempKeys.push(key)));

  const keys = _.uniq(tempKeys);
  const lengths = new Array<number>(keys.length).fill(1);
  const items = [];

  _.forEach(objects, (nodeInfo) => {
    const item = [];
    let idx = 0;
    _.forEach(keys, (key) => {
      item.push(nodeInfo[key] || '');
      lengths[idx] = 1;//Math.max(lengths[idx], item[idx].length);
      ++idx;
    });
    items.push(item);
  });

  const mdTextPad = [];

  const tableHeader = padArrayWithLength([keys], lengths);
  const tableHorizon = padArrayWithLength([lengths], lengths, '-');
  const tableBody = padArrayWithLength(items, lengths, ' ');

  const table = tableHeader.concat(tableHorizon).concat(tableBody);
  const stringArray = _.map(table, (v) => '| ' + v.join(' | ') + ' |');
  const envString = stringArray.join('\n');

  return envString;
};

export const createMdText = (objects: Object) => {
  let str = [];

  if (Array.isArray(objects)) {
    str.push('```javascript');
    _.forEach(objects, (value, key) => {
      str.push(`${value}`);
    });
    str.push('```');
    return str.join('\n');
  }

  if (!objects || Object.keys(objects).length === 0) return '';

  _.forEach(objects, (nodeInfo) => {
    if (nodeInfo.Name) {
      str.push(`### ${nodeInfo.Name}`);
    }

    str.push('```javascript');
    _.forEach(nodeInfo, (value, key) => {
      if (value === undefined || value.length === 0) return;
      if (key === 'Name') return;
      str.push(`${value}`);
    });
    str.push('```');
  });

  return str.join('\n');
};

export const replaceSection = (srcText: string, sectionName: string, replaceWith: string) => {
  if (!replaceWith) return srcText;

  const envMatchedSection = srcText.match(new RegExp(`\n(#+) ${sectionName}`));
  let sectionTitle = '\n\n## ' + sectionName;
  let prevSection = srcText;
  let nextSection = '';

  // 기본적으로 제일 뒤에 붙임.
  // 가운데에 # Environment Variables 가 있으면 replace.
  if (envMatchedSection) {
    sectionTitle = envMatchedSection[0];
    const sectionIndent = envMatchedSection[1];
    const startIdx = envMatchedSection.index;

    const nextMatchedSection = srcText.substring(startIdx + sectionTitle.length).match(new RegExp(`\n${sectionIndent} [A-z\s]+`));

    prevSection = srcText.substring(0, startIdx);
    if (nextMatchedSection) {
      const endIdx = nextMatchedSection ? nextMatchedSection.index : undefined;
      nextSection = srcText.substring(startIdx + endIdx + sectionTitle.length)
    }
  }

  return prevSection + sectionTitle + '\n\n' + replaceWith + '\n\n' + nextSection;
};